#ifndef LINT
static char *sccs_sid = "@(#)adecode.c	1.9 3/21/89 FAULKNLIB";
#endif

#include <stdio.h>

/*STARTDOC*/
/*  this module includes 
	adecode (query, choices, matches, base
	char *query, **choices;
	int *matches, base;
	fils in matches with indices of those who matched
	returns -1 if ambiguous, -2 if not found, index
	of choice if unambiguous
*/

/*ENDDOC*/
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

/*STARTDOC*/
int adecode (query, choices, matches, base)
/*  determines which members of choices that query matches,
    sets "matches" to contain those elements that matched.
    for numeric items, 'base' is subtracted from the user
	response before the item is placed in the array
    returns:   -1 if query is ambiguous
	       -2 if query matched no choices
	       #of choice if unambiguous
	       fills in matches array with things matched,
		   terminates list with "-"
*/
char *query, **choices;
int *matches, base;
/*ENDDOC*/
{   char stemp [400], *sptr;
    int num_matched, is_number, itemp, *matches_temp;

    is_number = 1;
    for (sptr = query; (*sptr != '\0') && is_number; sptr++)
        if ((*sptr < '0') || (*sptr > '9'))
	    if ((*sptr != ',') && (*sptr != '-') && (*sptr > ' '))
		is_number = 0;

    if (matches == NULL)
    {	itemp = calloc (300, sizeof (int));
	matches_temp = matches = (int *) itemp;
    }
    if (is_number)
       num_matched = adec_num_exp (query, count_strings (choices), 
	   matches, base);
    else
    {   sprintf (stemp, "^%s", query);
	num_matched = do_matches (stemp, choices, matches);
	if (num_matched == 0)
	    num_matched = do_matches (query, choices, matches);
    }

    if (matches_temp)
	free (matches);

    switch (num_matched)
    {   case 0:   /*  none found  */
	    return (-2);
	    break;
	case 1:   /*  exactly one match  */
	    return (*matches);
	    break;
	default:  /*  more than one match  */
	    return (-1);
	    break;
    }
    return (-1);
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

do_matches (query, choices, matches)
char *query, **choices;
int *matches;
{   char *re_result, *re_comp ();
    int num_matched, index;

    if (re_result = re_comp (query))
    {   fprintf (stderr, "Bad regex \"%s\", %s\n",
	    query, re_result);
	*matches = -1;
	return (0);
    }
    num_matched = 0;
    for (index = 0; *choices && **choices; index++)
    {   if (re_exec (*choices) == 1)
	{   matches [num_matched] = index;
	    num_matched++;
	}
	choices++;
    }
    matches [num_matched] = -1;
    return (num_matched);
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

count_strings (str)
char **str;
{   int i;

    for (i = 0; (str [i] != NULL) && (*(str [i]) != '\0'); i++);
    return (i);
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

adec_num_exp (query, num_choices, matches, base)
char *query;
int num_choices, *matches, base;
{   char *ptr;
    int num1, num2, index, itemp, num_matches;

    num_matches = 0;
    for (ptr = query; *ptr != '\0';)
    {   eat_num (&num1, &ptr);
	switch (*ptr)
	{   case ',':
	    case ' ':
	    case '\0':
		itemp = num1 - base;
		if ((itemp < 0) || (itemp > num_choices))
		{   printf ("%d out of range\n", itemp);
		}
		else
		{   matches [num_matches] = num1 - base;
		    num_matches++;
		}
		break;
	    case '-':
		ptr++;
		eat_num (&num2, &ptr);
		num1 -= base;
		num2 -= base;
		itemp = 1;
		if (num1 < 0)
		    num1 = 0;
		if (num2 < 0)
		    itemp = 0;
		if (num1 > num_choices)
		    itemp = 0;
		if (num2 >= num_choices)
		    num2 = num_choices - 1;
		if (num1 > num2)
		    itemp = 0;
		if (itemp)
		    for (index = num1; index <= num2; index++)
		    {   itemp = index;
			matches [num_matches] = index;
			num_matches++;
		    }
		else
		    printf ("Bad range: %d to %d\n", num1 + base,
			num2 + base);
		break;
	    default:
		fprintf (stderr, "Problem: %d \"%s\", unk div\n", 
		    num1, ptr);
		break;
	}
	if (*ptr != '\0')
	    ptr++;
    }
    matches [num_matches] = -1;
    return (num_matches);
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

eat_num (num, str)
int *num;
char **str;
{   *num = 0;
    while ((**str >= '0') && (**str <= '9'))
    {   *num = *num * 10 + **str - '0';
	(*str)++;
    }
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

