#ifndef LINT
static char *sccs_sid = "@(#)compensated.c	1.8 6/12/97 MASE";
#endif

#include "edseq.h"
#include "extern.h"
#include <strings.h>

#define FORWARD 1
#define BACKWARD -1

ins_comp_forward (current_y, current_x, mult)
int *current_y, *current_x, mult;
{   
    ins_1 (*current_y, *current_x, mult, current_key);
    compensate (*current_y, *current_x, FORWARD, -1 * mult);
    dirty_x = 1;
}

ins_comp_backward (current_y, current_x, mult)
int *current_y, *current_x, mult;
{
    ins_1 (*current_y, *current_x, mult, current_key);
    compensate (*current_y, *current_x, BACKWARD, -1 * mult);
    dirty_x = 1;
}

del_comp_forward (current_y, current_x, mult)
int *current_y, *current_x, mult;
{
    del_1 (*current_y, *current_x, mult);
    compensate (*current_y, *current_x, FORWARD, mult);
    dirty_x = 1;
}

del_comp_backward (current_y, current_x, mult)
int *current_y, *current_x, mult;
{
    del_1 (*current_y, *current_x - mult + 1, mult);
    compensate (*current_y, *current_x, BACKWARD, mult);
    dirty_x = 1;
}

compensate (which, where, direction, num)
int which, where, direction, num;
{  
    if (!num)
	return 0;
    
    if (direction == FORWARD)
	return (forw_compensate (which, where, num));
    else
	return (back_compensate (which, where, num));
}


forw_compensate (which, where, num)
int which, where, num;
{   char *seq, *hit;
    int pos, indx, tpos, num_to_delete, dot;

    seq = entry [which] -> seq;
    hit = index (&(seq [where]), '.');
    if (!hit)
      return;  /*  no period, nothing to compensate  */
    pos = hit - seq - 1;  /*  insert before the dot  */
    dot = pos + num;

    if (num > 0)
	ins_1 (which, pos + 1, num, '-');
    else  {
        for (tpos = pos; seq [tpos] == '-'; tpos--);
	tpos++;  /*  tpos to last ``-''  */
	num_to_delete = pos - tpos + 1;
	if (num_to_delete > -num)
	  num_to_delete = -num;
	del_1 (which, tpos, num_to_delete);
	num += num_to_delete;
	if (num < 0)  {
	    for (indx = 0; indx < num_entries; indx++)
		if (indx != which)
		    ins_1 (indx, dot + 1, -1 * num, '-');
	    dirty_y = 1;
	}
    }
}


back_compensate (which, where, num)
int which, where, num;
{   char *seq;
    int pos, indx, tpos, num_to_delete;

    seq = entry [which] -> seq;
    for (pos = where; (pos > 0) && (seq [pos] != '.'); pos--);

    if (pos == 0) {  /*  doing something at the beginning of buffer */
	if (num > 0)  {
	   ins_1 (which, 0, num, '-'); 
	}
	else  {
	    for (tpos = 0; seq [tpos] == '-'; tpos++);
	    if (tpos)  {
	      num_to_delete = tpos;
	      if (num_to_delete > -num)
		num_to_delete = -num;
	      del_1 (which, 0, num_to_delete);
	      num += num_to_delete;
	    }
	    for (indx = 0; indx < num_entries; indx++)
		if (indx != which)
		    ins_1 (indx, 0, -1 * num, '-');
	    dirty_y = 1;
	}
    }
    else  {  /*  doing something in the middle of the buffer  */
	if (num > 0)  {
	    ins_1 (which, pos + 1, num, '-');
	}
	else  {
	    for (tpos = pos + 1; seq [tpos] == '-'; tpos++);
	    tpos--;
	    num_to_delete = tpos - pos;
	    if (num_to_delete > -num)
	      num_to_delete = -num;
	    del_1 (which, pos + 1, num_to_delete);
	    num += num_to_delete;
	    if (num)  {
		for (indx = 0; indx <  num_entries; indx++)
		    if (indx != which)
			ins_1 (indx, pos + 1, -1 * num, '-');
		dirty_y = 1;
	    }
	}
    }
}




check_anchor ()
/*  sanity check - do all sequences have same # of anchors??  */
/*  return 1 if o.k., 0 if bad  */
{   int indx, number, this;

    if (num_entries == 1)
      return (1);
    number = count_anchors (0);
    for (indx = 0; indx < num_entries; indx++)
      if (number != (this = count_anchors (indx)))  {
	move (LINES - 2, 0);
	printw ("Anchor counting: #1 had %d != #%d had %d",
		number, indx + 1, this);
	Pause (0);
	return (0);
      }
    return (1);
}


count_anchors (num)
int num;
{   char *ptr;
    int indx;

    for (indx = 0, ptr = (entry [num] -> seq) - 1;
	 ptr = index (ptr + 1, '.'); indx++);
    return (indx);
}

sync_anchor ()
{   int hits [NUMBER_SEQ], indx, farthest, this, next;

    if (!check_anchor ())
      return;
    
    this = 0;
    do  {
      farthest = -1;
      for (indx = 0; indx < num_entries; indx++)  {
	  if (this >= entry [indx] -> length)
	    next = -1;
	  else  {
	    next = (int) index (&((entry [indx] -> seq) [this]), '.');
	    if (next)  {
	      next = next - (int) entry [indx] -> seq;
	      if (next > farthest)
		farthest = next;
	    }
	    else
	      next = -1;
	  }
	  hits [indx] = next;
      }
      for (indx = 0; indx < num_entries; indx++)
	if (hits [indx] != farthest)
	  ins_1 (indx, hits [indx], farthest - hits [indx], '-');

      this = farthest + 1;
    }   while (farthest >= 0);
    dirty_y = 1;
}

/* (PWK - 09/06/12) */
/* sync's anchors only in specified list of sequences */
sync_anchor2 (sq_list)
int *sq_list;
{   int hits [NUMBER_SEQ], *indx, farthest, this, next;

    this = 0;
    do  {
      farthest = -1;
      for (indx = sq_list; *indx >= 0; indx++)  {
        if (this >= entry [*indx - 1] -> length)
          next = -1;
        else  {
          next = (int) index (&((entry [*indx - 1] -> seq) [this]), '.');
          if (next)  {
            next = next - (int) entry [*indx - 1] -> seq;
            if (next > farthest)
              farthest = next;
          }
          else
            next = -1;
        }
        hits [*indx - 1] = next;
      }
      for (indx = sq_list; *indx >= 0; indx++)
        if (hits [*indx - 1] != farthest)
          ins_1 (*indx - 1, hits[*indx - 1], farthest - hits[*indx - 1], '-');

      this = farthest + 1;
    }   while (farthest >= 0);
    dirty_y = 1;
}


delete_anchor (current_y, current_x)
int *current_y, *current_x;
{   static char *list [200] = { NULL };
    char stemp [200], **ptr, *new_string ();
    int where, num_not_found, del_list [NUMBER_SEQ], num_found, indx;
    sq *this;

    sprintf (stemp, "%d", *current_x + 1);
    if (list [0])  {
      for (ptr = list; *ptr; ptr++);
      ptr--;
      if (strcmp (stemp, *ptr))  {
	ptr++;
	*ptr = new_string (stemp);
      }
    }
    else  {
      list [0] = new_string (stemp);
    }
    win_prompt (stdscr, LINES - 2, "Delete anchor at which column",
		stemp, list, 0, "AnchorWhere");
    where = -1;
    sscanf (stemp, "%d", &where);
    if (where <= 0)  {
      move (LINES - 2, 0);
      printw ("%d is not a valid position", where);
      Pause (0);
      return;
    }
    where--;
    for (num_found = num_not_found = indx = 0; indx < num_entries; indx++)  {
      this = entry [indx];
      if ((this -> length < where) || ((this -> seq) [where] != '.'))  {
	num_not_found++;
	move (LINES - 2, 0);
	printw ("No anchor at %d in locus #%d (%s)", where + 1, indx + 1,
		this -> locus_name);
	Pause (0);
      }
      else {
	del_list [num_found++] = indx;
      }
    }
    if (num_not_found)  {
      move (LINES - 2, 0);
      printw ("%d sequences did not have this anchor", num_not_found);
      Pause (0);
    }
    for (indx = 0; indx < num_found; indx++)  {
      del_1 (del_list [indx], where, 1);
    }
    dirty_x = dirty_y = 1;
}

strip_anchor (current_y, current_x)
int *current_y, *current_x;
{   int indx, where;
    sq *this;
    char *seq;

    move (LINES - 2, 0);
    printw ("Stripping all anchors from all sequences, ^C to abort");
    Pause (0);
    clean_bottom ();
    dirty_x = dirty_y = 1;
    for (indx = 0; indx < num_entries; indx++)  {
      if ((indx < 10) || (!((indx + 1) % 10)))  {
	move (LINES - 2, 0);
	printw ("%5d", indx + 1);
	refresh ();
      }
      seq = (this = entry [indx]) -> seq;
      for (where = 0; where < this -> length; where++)
	if (seq [where] == '.')
	  del_1 (indx, where, 1);
    }
    clean_bottom ();
}

/* (PWK - 09/06/12) */
/* strip anchors only from specified list of sequences */
strip_anchor2 (sq_list)
int *sq_list;
{   int *indx, where;
    sq *this;
    char *seq;

    dirty_x = dirty_y = 1;
    for (indx = sq_list; *indx >= 0; indx++)  {
      this = entry [*indx - 1];
      seq = this -> seq;
      for (where = 0; where < this -> length; where++)
        if (seq [where] == '.')
          del_1 (*indx - 1, where, 1);
    }
}


create_anchor (current_y, current_x)
int *current_y, *current_x;
{   int where, indx;
    char stemp [200], stemp2 [200];
    static char *history [3];

    sprintf (stemp2, "%d", *current_x + 1);
    history [0] = stemp2;
    history [1] = NULL;

    win_prompt (stdscr, LINES - 2, 
	"Create an anchor at which column", stemp, history, 0,
         "AnchorWhere");
    where = 0;
    sscanf (stemp, "%d", &where);
    where--;
    if ((where < 0) || (where > MAX_SEQ)) {
          move (LINES - 2, 0);
	  printw ("%d: Out of range", where + 1);
	  Pause (0);
    }
    for (indx = 0; indx < num_entries; indx++) {
      if (where < entry [indx] -> length)
	ins_1 (indx, where, 1, '.');
      else
	ins_1 (indx, entry [indx] -> length, 1, '.');
    }
    dirty_x = dirty_y = 1;
}



